@@ -11,7 +11,7 @@
 //!  dead simple
 //!   static site generator.
 //!
-//! #Features
+//! # Features
 //! Currently, Cargo doesn't support binary-only dependencies. As such, these dependencies are behind
 //! the `bin` feature gate, which is enabled by default. When using Blades as a library, they are not
 //! necessary, so it is recommended to import blades with `default_features = false`.
@@ -24,8 +24,8 @@ mod taxonomies;
 mod types;
 
 pub use config::Config;
-pub use page::Page;
+pub use page::{Page, Pages, Picture};
 pub use sources::{Parser, Source, Sources};
-pub use tasks::{cleanup, colocate_assets, render_meta};
-pub use taxonomies::Taxonomy;
-pub use types::{MutSet, Templates};
+pub use tasks::{cleanup, colocate_assets, load_templates, render_meta};
+pub use taxonomies::{Taxonomies, Taxonomy};
+pub use types::{Ancestors, Any, DateTime, HashMap, MutSet};
