@@ -6,40 +6,39 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with Blades.  If not, see <http://www.gnu.org/licenses/>
-use crate::config::{Config, TEMPLATE_DIR};
-
 use beef::lean::Cow;
 use chrono::{DateTime as CDateTime, Datelike, FixedOffset, NaiveDate, NaiveDateTime, Timelike};
 use ramhorns::encoding::Encoder;
 use ramhorns::traits::ContentSequence;
-use ramhorns::{Content, Ramhorns, Section, Template};
+use ramhorns::{Content, Section};
 use serde::de::{self, Deserialize, Deserializer, Visitor};
 
+use std::borrow::Borrow;
 use std::collections::HashSet;
 use std::fmt;
-use std::path::{is_separator, Path, PathBuf};
+use std::hash::Hash;
+use std::ops::{Deref, DerefMut};
+use std::path::{is_separator, PathBuf};
 use std::time::SystemTime;
 
-pub(crate) type HashMap<K, V> = std::collections::HashMap<K, V, ahash::RandomState>;
 /// A set of all rendered paths. Behind a mutex, so it can be written from multiple threads.
 pub type MutSet<T = PathBuf> = parking_lot::Mutex<HashSet<T, ahash::RandomState>>;
 
-/// Aggregation of all the templets of the site's theme and its template dir.
-pub struct Templates {
-    templates: Option<Ramhorns>,
-    theme: Option<Ramhorns>,
-}
+/// A hash map wrapper that can render fields directly by the hash.
+#[derive(Clone, serde::Deserialize, serde::Serialize)]
+#[serde(transparent)]
+pub struct HashMap<K: Hash + Eq, V>(pub(crate) hashbrown::HashMap<K, V, fnv::FnvBuildHasher>);
 
-/// A wrapper around the `choron::NaiveDateTime`, used for rendering of dates.
+/// A wrapper around the `chrono::NaiveDateTime`, used for rendering of dates.
 #[derive(Clone, Copy, Debug, PartialEq, Eq, PartialOrd, Ord, serde::Serialize)]
 #[serde(transparent)]
-pub struct DateTime(NaiveDateTime);
+pub struct DateTime(pub NaiveDateTime);
 
 /// A wrapper around a `str` representing path, used to derive `Content` implementation
 /// that acts like an iterator over the path segmets.
 #[derive(serde::Deserialize, serde::Serialize)]
 #[serde(transparent)]
-pub(crate) struct Ancestors<'a>(#[serde(borrow)] pub(crate) Cow<'a, str>);
+pub struct Ancestors<'a>(#[serde(borrow)] pub Cow<'a, str>);
 
 /// One segment of a path.
 #[derive(Content)]
@@ -55,50 +54,21 @@ struct Segment<'a>(
 /// A sum of all the types that can be used in a TOML file.
 #[derive(Clone, serde::Deserialize, serde::Serialize)]
 #[serde(untagged)]
-pub(crate) enum Any<'a> {
+pub enum Any<'a> {
+    /// A string.
     String(#[serde(borrow)] Cow<'a, str>),
+    /// A number.
     Number(f64),
+    /// A boolean value.
+    Bool(bool),
+    /// Date and time data.
     DateTime(DateTime),
+    /// A list.
     List(Vec<Any<'a>>),
+    /// A key-value map.
     Map(HashMap<&'a str, Any<'a>>),
 }
 
-impl Templates {
-    /// Load the templates from the directories specified by the config.
-    #[inline]
-    pub fn load(config: &Config) -> Result<Self, ramhorns::Error> {
-        Ok(Self {
-            templates: if Path::new(TEMPLATE_DIR).exists() {
-                Some(Ramhorns::from_folder(TEMPLATE_DIR)?)
-            } else {
-                None
-            },
-            theme: if !config.theme.is_empty() {
-                let mut theme_path =
-                    Path::new(config.theme_dir.as_ref()).join(config.theme.as_ref());
-                theme_path.push(TEMPLATE_DIR);
-                if theme_path.exists() {
-                    Some(Ramhorns::from_folder(theme_path)?)
-                } else {
-                    None
-                }
-            } else {
-                None
-            },
-        })
-    }
-
-    /// Get one template with the given name or return an error.
-    #[inline]
-    pub fn get(&self, name: &str) -> Result<&Template<'static>, ramhorns::Error> {
-        self.templates
-            .as_ref()
-            .and_then(|t| t.get(name))
-            .or_else(|| self.theme.as_ref().and_then(|t| t.get(name)))
-            .ok_or_else(|| ramhorns::Error::NotFound(name.into()))
-    }
-}
-
 impl<'a> Content for Ancestors<'a> {
     #[inline]
     fn is_truthy(&self) -> bool {
@@ -169,10 +139,25 @@ impl<'a> From<Cow<'a, str>> for Ancestors<'a> {
     }
 }
 
+#[inline]
+fn content_without_paragraphs<E: Encoder>(source: &str, encoder: &mut E) -> Result<(), E::Error> {
+    use pulldown_cmark::{Event, Tag};
+    let parser =
+        pulldown_cmark::Parser::new_ext(source, pulldown_cmark::Options::all()).filter(|event| {
+            !matches!(
+                event,
+                Event::Start(Tag::Paragraph) | Event::End(Tag::Paragraph),
+            )
+        });
+    let processed = cmark_syntax::SyntaxPreprocessor::new(parser);
+    encoder.write_html(processed)
+}
+
 impl<'a> Content for Any<'a> {
     #[inline]
     fn is_truthy(&self) -> bool {
         match self {
+            Any::Bool(b) => *b,
             Any::List(vec) => !vec.is_empty(),
             Any::Map(map) => !map.is_empty(),
             Any::String(s) => !s.is_empty(),
@@ -184,7 +169,8 @@ impl<'a> Content for Any<'a> {
     #[inline]
     fn render_escaped<E: Encoder>(&self, encoder: &mut E) -> Result<(), E::Error> {
         match self {
-            Any::String(s) => s.render_cmark(encoder),
+            Any::Bool(b) => b.render_escaped(encoder),
+            Any::String(ref s) => content_without_paragraphs(s, encoder),
             Any::Number(n) => n.render_escaped(encoder),
             Any::DateTime(dt) => dt.render_escaped(encoder),
             Any::List(vec) => vec.render_escaped(encoder),
@@ -195,6 +181,7 @@ impl<'a> Content for Any<'a> {
     #[inline]
     fn render_unescaped<E: Encoder>(&self, encoder: &mut E) -> Result<(), E::Error> {
         match self {
+            Any::Bool(b) => b.render_unescaped(encoder),
             Any::String(s) => s.render_unescaped(encoder),
             Any::Number(n) => n.render_unescaped(encoder),
             Any::DateTime(dt) => dt.render_unescaped(encoder),
@@ -283,6 +270,7 @@ impl<'a> Content for Any<'a> {
 }
 
 impl DateTime {
+    /// The date and time right now.
     pub fn now() -> Self {
         SystemTime::now().into()
     }
@@ -359,7 +347,6 @@ impl Content for DateTime {
 // Toml crate currently doesn't supprot deserializing dates into types other than String,
 // so an ugly hack based on its `Deserializer` private fields needs to be used.
 const FIELD: &str = "$__toml_private_datetime";
-const NAME: &str = "$__toml_private_Datetime";
 
 impl<'de> Deserialize<'de> for DateTime {
     fn deserialize<D>(deserializer: D) -> Result<DateTime, D::Error>
@@ -431,8 +418,7 @@ impl<'de> Deserialize<'de> for DateTime {
             }
         }
 
-        static FIELDS: [&str; 1] = [FIELD];
-        deserializer.deserialize_struct(NAME, &FIELDS, DateTimeVisitor)
+        deserializer.deserialize_str(DateTimeVisitor)
     }
 }
 
@@ -442,3 +428,149 @@ impl From<SystemTime> for DateTime {
         DateTime(time.naive_utc())
     }
 }
+
+impl<K: Borrow<str> + Hash + Eq, V: Content> Content for HashMap<K, V> {
+    #[inline]
+    fn is_truthy(&self) -> bool {
+        !self.is_empty()
+    }
+
+    /// Render a section with self.
+    #[inline]
+    fn render_section<C, E>(&self, section: Section<C>, encoder: &mut E) -> Result<(), E::Error>
+    where
+        C: ContentSequence,
+        E: Encoder,
+    {
+        if self.is_truthy() {
+            section.with(self).render(encoder)
+        } else {
+            Ok(())
+        }
+    }
+
+    #[inline]
+    fn render_field_escaped<E>(
+        &self,
+        hash: u64,
+        _name: &str,
+        encoder: &mut E,
+    ) -> Result<bool, E::Error>
+    where
+        E: Encoder,
+    {
+        match self.raw_entry().from_hash(hash, |_| true) {
+            Some((_, v)) => v.render_escaped(encoder).map(|_| true),
+            None => Ok(false),
+        }
+    }
+
+    #[inline]
+    fn render_field_unescaped<E>(
+        &self,
+        hash: u64,
+        _name: &str,
+        encoder: &mut E,
+    ) -> Result<bool, E::Error>
+    where
+        E: Encoder,
+    {
+        match self.raw_entry().from_hash(hash, |_| true) {
+            Some((_, v)) => v.render_unescaped(encoder).map(|_| true),
+            None => Ok(false),
+        }
+    }
+
+    #[inline]
+    fn render_field_section<C, E>(
+        &self,
+        hash: u64,
+        _name: &str,
+        section: Section<C>,
+        encoder: &mut E,
+    ) -> Result<bool, E::Error>
+    where
+        C: ContentSequence,
+        E: Encoder,
+    {
+        match self.raw_entry().from_hash(hash, |_| true) {
+            Some((_, v)) => v.render_section(section, encoder).map(|_| true),
+            None => Ok(false),
+        }
+    }
+
+    #[inline]
+    fn render_field_inverse<C, E>(
+        &self,
+        hash: u64,
+        _name: &str,
+        section: Section<C>,
+        encoder: &mut E,
+    ) -> Result<bool, E::Error>
+    where
+        C: ContentSequence,
+        E: Encoder,
+    {
+        match self.raw_entry().from_hash(hash, |_| true) {
+            Some((_, v)) => v.render_inverse(section, encoder).map(|_| true),
+            None => Ok(false),
+        }
+    }
+}
+
+impl<K: Hash + Eq, V> Default for HashMap<K, V> {
+    #[inline]
+    fn default() -> Self {
+        Self(Default::default())
+    }
+}
+
+impl<K: Hash + Eq, V> Deref for HashMap<K, V> {
+    type Target = hashbrown::HashMap<K, V, fnv::FnvBuildHasher>;
+
+    #[inline]
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
+impl<K: Hash + Eq, V> DerefMut for HashMap<K, V> {
+    #[inline]
+    fn deref_mut(&mut self) -> &mut Self::Target {
+        &mut self.0
+    }
+}
+
+impl<K: Hash + Eq, V> HashMap<K, V> {
+    #[inline]
+    pub(crate) fn is_empty(&self) -> bool {
+        self.0.is_empty()
+    }
+}
+
+#[cfg(test)]
+mod test {
+    #[test]
+    fn simple_render_hash_map() {
+        use super::HashMap;
+
+        let source = "<title>{{title}}</title><h1>{{ title }}</h1><div>{{body}}</div>";
+        let tpl = ramhorns::Template::new(source).unwrap();
+
+        let mut map = HashMap::default();
+
+        map.insert("title", "Hello, Ramhorns!");
+        map.insert(
+            "body",
+            "This is a test of rendering a template with a HashMap Content!",
+        );
+
+        let rendered = tpl.render(&map);
+
+        assert_eq!(
+            &rendered,
+            "<title>Hello, Ramhorns!</title><h1>Hello, Ramhorns!</h1>\
+         <div>This is a test of rendering a template with a HashMap Content!</div>"
+        );
+    }
+}
